unit RestFrameClasses;

interface

uses
  Classes;

type
  TRestFormat = (rfHtml, rfXml, rfPdf, rfUnknown);

  TRestWorker = class (TPersistent)
  public
    function Process (path, query: TStrings; format: TRestFormat): string; virtual; abstract;
    function Xslt (xmlDoc: string; path: TStrings): string;
    function TypeNameFromPath (path: TStrings): string;
  end;

  TDefaultWorker = class (TRestWorker)
  public
    function Process(path: TStrings; query: TStrings;
      format: TRestFormat): string; override;
    function TableToData (tablename: string; format: TRestFormat): string;
    function RecordToData (tablename: string; recId: string; format: TRestFormat): string;
    function KeyFieldForTable (tablename: string): string;
  end;

implementation

uses
  RestFrameData, StrUtils, DB, SysUtils, XslProcess;

// XML support functions

function MakeXmlStr (nodeName, nodeValue: string; attrList: string = ''): string;
begin
  Result := '<' + nodeName +
    IfThen (attrList <> '', ' ' + attrList, '') + '>' +
    nodeValue +
    '</' + nodeName + '>';
end;

function MakeXmlAttribute (attrName, attrValue: string): string;
begin
  Result := attrName + '="' + attrValue + '"';
end;

function FieldsToXml (data: TDataSet; rootName: string): string;
var
  i: Integer;
begin
  Result := '<' + rootName + '>' + sLineBreak;;
  for i := 0 to data.FieldCount - 1 do
    Result := Result + '  ' + MakeXmlStr (
      LowerCase (data.Fields[i].FieldName),
      data.Fields[i].AsString) + sLineBreak;
  Result := Result + '</' + rootName + '>' + sLineBreak;;
end;

function DataSetToXml (dataset: TDataset; tablename: string): string;
begin
  Result := '<' + tablename  + '_table>' + sLineBreak;
  dataset.First;
  while not dataset.EOF do
  begin
    Result := Result + '  ' + FieldsToXml (dataset, tablename) + sLineBreak;
    dataset.Next;
  end;
  Result := Result + '</' + tablename  + '_table>';
end;

{ TRestWorker }

function TRestWorker.TypeNameFromPath(path: TStrings): string;
begin
  Result := path[0];
  if Result = '' then
    Result := 'default';
end;

function TRestWorker.Xslt(xmlDoc: string; path: TStrings): string;
var
  xslpro: TXslProcess;
begin
  xslpro := TXslProcess.Create;
  xslpro.LoadXml(xmlDoc);
  xslPro.LoadXslFile(TypeNameFromPath (path) + '.xsl');
  xslPro.ApplyXslToXml;
  Result := xslPro.GetResult;
end;

{ TDefaultWorker }

function TDefaultWorker.KeyFieldForTable(tablename: string): string;
begin
  // use an external configuration mapper
  if tablename= 'employee' then
    Result := 'emp_no'
  else if tablename = 'department' then
    Result := 'dept_no'
  else
    Result := 'id';
end;

function TDefaultWorker.Process(path, query: TStrings;
  format: TRestFormat): string;
begin
  if Path[0] <> '' then
    case Path.Count of
      1: Result :=
        TableToData (path[0], format);
      2: Result :=
        RecordToData (path[0], path[1], format);
    end;
end;

function TDefaultWorker.RecordToData(tablename, recId: string;
  format: TRestFormat): string;
begin
  DataModule1.SQLDataSet1.Close;
  DataModule1.SQLDataSet1.CommandText :=
    'select * from ' + tablename +
    ' where ' + KeyFieldForTable (tablename) +
      ' = ' + recId;
  DataModule1.SQLDataSet1.Open;

  Result := FieldsToXml (DataModule1.SQLDataSet1, tablename);
end;

function TDefaultWorker.TableToData(tablename: string;
  format: TRestFormat): string;
begin
  DataModule1.SQLDataSet1.Close;
  DataModule1.SQLDataSet1.CommandText :=
    'select * from ' + tablename;
  DataModule1.SQLDataSet1.Open;

  Result := DataSetToXml (
    DataModule1.SQLDataSet1, tablename);
end;

end.
