unit RestFrameDM;

interface

uses
  SysUtils, Classes, HTTPApp;

type
  TWebModule4 = class(TWebModule)
    procedure WebModuleBeforeDispatch(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
  private
    procedure ProcessPath(strPath: string; sList: TStringList);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  WebModule4: TWebModule4;

implementation

uses
  WebReq, RestFrameClasses, xslprocess;

{$R *.dfm}

procedure TWebModule4.ProcessPath (strPath: string; sList: TStringList);
begin
  sList.LineBreak := '/';
  sList.Text := strPath;
end;

procedure TWebModule4.WebModuleBeforeDispatch(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  strPath, strResp, ext: string;
  sList: TStringList;
  worker: TRestWorker;
  format: TRestFormat;
begin
  strPath := Request.InternalPathInfo; //PathInfo;

  // normalize path
  if strPath = '' then
    strPath := '/';
  if (strPath[1] = '/') and (Length (strPath) > 1) then
    strPath := copy (strPath, 2, maxint);

  strResp := '';
  sList := TStringList.Create;
  try
    ProcessPath (strPath, sList);

    // debug
    // for str in sList do
    //   strResp := strResp + '<li>' + str;

    // grab extension, figure out type
    ext := ExtractFileExt (sList[sList.Count-1]);
    sList[sList.Count-1] := ChangeFileExt (sList[sList.Count-1], '');
    if (upperCase (ext) = '.HTML') or (ext = '') then
      format := rfHtml
    else if upperCase (ext) = '.HTM' then
      format := rfHtml
    else if upperCase (ext) = '.XML' then
      format := rfXml
    else if upperCase (ext) = '.PDF' then
      format := rfPdf
    else
      format := rfUnknown;

    worker := TDefaultWorker.Create;
    try
      strResp := strResp + worker.Process(sList, Request.QueryFields, format);

      case format of
        rfHtml: Response.Content :=
          Worker.Xslt (strResp, sList);
        rfXml: Response.Content := strResp;
        rfPdf: Response.Content := strResp; // todo
        rfUnknown: Response.Content := strResp; // error?
      end;

      case format of
        rfHtml: Response.ContentType := 'text/html';
        rfXml: Response.ContentType := 'text/xml';
        rfPdf: Response.ContentType := 'application/pdf';
        rfUnknown: Response.ContentType := 'text/xml';
      end;

    finally
      worker.Free;
    end;
  finally
    sList.Free;
  end;
  Handled := True;
end;

initialization
  if WebRequestHandler <> nil then
    WebRequestHandler.WebModuleClass := TWebModule4;

end.
