unit XmlHelper;

interface

type
  // XML encoding enumeration
  TXMLEncoding =
    (seUTF8, // UTF8  - the default
    seUTF16LE, // UTF16 - little endian (Intel)
    seUTF16BE, // UTF16 - big endian
    seISO88591); // ISO-8859-1 - Latin 1

// add the <?xml with the proper encoding
function CheckXmlEncoding(const xmlString: string; encoding: TXmlEncoding = seISO88591): string;
function EncodingString(encoding: TXmlEncoding): string;
function FixPathDelimiters (strPath: string; delim: char): string;


implementation

uses
  SysUtils;

function FixPathDelimiters (strPath: string; delim: char): string;
var
  i: Integer;
begin
  result := '';
  if delim = #0 then
    delim := pathDelim;
  for i := 1 to Length (strPath) do
    if( (strPath[i] = '/') or (strPath[i] = '\') ) then
      Result := Result + delim
    else
      Result := Result + strPath[i];
end;

function CheckXmlEncoding(const xmlString: string; encoding: TXmlEncoding): string;
var
  nPos: Integer;
begin
  nPos := Pos('<?xml', xmlString);
//ag
//  if nPos > 0 then
  if nPos = 1 then
    Result := EncodingString(encoding) + sLineBreak +
      Copy(xmlString, Pos('?>', xmlString) + 2, maxint)
  else
    Result := EncodingString(encoding) + sLineBreak + xmlString;
end;

function EncodingString(encoding: TXmlEncoding): string;
begin
  case encoding of
    seUTF8: Result := '<?xml version="1.0" encoding="UTF-8"?>';
    seUTF16LE: Result := '<?xml version="1.0" encoding="UTF-16"?>';
    seUTF16BE: Result := '<?xml version="1.0" encoding="UTF-16"?>';
    seISO88591: Result := '<?xml version="1.0" encoding="ISO-8859-1"?>';
  end;
end;

end.
