unit XslProcess;

interface

uses
  LibxmlHelper, libxml2, libxslt;

type
  TXslProcess = class
  private
    fXmlDoc, fxmlResDoc: xmlDocPtr;
    fXslDoc: xsltStylesheetPtr;
  public
    procedure LoadXsl (const xslText: string; const url: string = '');
    procedure LoadXslFile (const xslFileName: string; const url: string = '');
    procedure LoadXml (const xmlText: string);
    procedure LoadXmlFile(const xmlFileName: string);

    function applyXPathToXml (const xpath: string; nodeSetCopy: Boolean = False): string;
    // function applyXPathToXsl (xpath: string; nodeSetCopy: Boolean = False): string;
    function applyXPathToResult (const xpath: string; nodeSetCopy: Boolean = False): string;

    procedure ApplyXslToXml;
    function GetResult (output: TXslOutput = xoXML): string;

    destructor Destroy; override;
  end;

  function XmlIsWellFormed (const strXmlText: string): Boolean;

implementation

uses
  Classes, SysUtils;

{ TGeoXslProcess }

function TXslProcess.applyXPathToResult(const xpath: string;
  nodeSetCopy: Boolean): string;
begin
  if not Assigned (fxmlResDoc) then
    raise Exception.Create('TGeoXslProcess.applyXPathToResult: xmlresult not initialized');
  Result := GlobalXslRendering.applyXPath (fxmlResDoc, xpath, nodeSetCopy);
end;

function TXslProcess.applyXPathToXml(const xpath: string;
  nodeSetCopy: Boolean): string;
begin
  if not Assigned (fXmlDoc) then
    raise Exception.Create('TGeoXslProcess.applyXPathToXml: xmldocument not initialized');
  Result := GlobalXslRendering.applyXPath (fXmlDoc, xpath, nodeSetCopy);
//  Result := FromUTF8 (Result); // convert to windows characters
end;

{function TGeoXslProcess.applyXPathToXsl(xpath: string;
  nodeSetCopy: Boolean): string;
begin
  if not Assigned (fXslDoc) then
    raise Exception.Create('TGeoXslProcess.applyXPathToXsl: xsldocument not initialized');
  ???
end;}

procedure TXslProcess.ApplyXslToXml;
begin
  if not Assigned (fXmlDoc) then
    raise Exception.Create('TGeoXslProcess.ApplyXslToXml: xmldocument not initialized');
  if not Assigned (fXslDoc) then
    raise Exception.Create('TGeoXslProcess.ApplyXslToXml: xsldocument not initialized');

  fxmlResDoc := GlobalXslRendering.ApplyXslToXml(fXmlDoc, fXslDoc);
end;

destructor TXslProcess.Destroy;
begin
  if Assigned(fXmlDoc) then xmlFreeDoc(fXmlDoc);
  if Assigned(fxmlResDoc) then xmlFreeDoc(fxmlResDoc);
  if Assigned(fXslDoc) then xsltFreeStylesheet(fXslDoc);

  inherited;
end;

function TXslProcess.GetResult(output: TXslOutput): string;
begin
  if not Assigned (fxmlResDoc) then
    raise Exception.Create('TGeoXslProcess.GetResult: xmlresult not initialized');

  Result := GlobalXslRendering.dumpDocument(fxmlResDoc, output);
end;

procedure TXslProcess.LoadXml(const xmlText: string);
begin
  fXmlDoc := GlobalXslRendering.parseDocument (xmlText);
end;

procedure TXslProcess.LoadXsl(const xslText, url: string);
begin
  fXslDoc := GlobalXslRendering.parseStyleSheet (xslText, url);
end;

procedure TXslProcess.LoadXslFile(const xslFileName, url: string);
begin
  LoadXsl (GlobalXslRendering.GetXslFile(XslFileName), url);
end;

procedure TXslProcess.LoadXmlFile(const xmlFileName: string);
var
  aStream: TFileStream;
  xmlText: string;
begin
  if not FileExists(xmlfilename) then
    raise Exception.Create(
      'XML file "' + xmlfilename + '" not found');

  // carica l'XSL da uno stream
  aStream := TFileStream.Create(xmlfilename,
    fmOpenRead + fmShareDenyWrite);
  try
    SetLength(xmlText, aStream.size);
    aStream.Read(xmlText[1], aStream.size);
    xmlText := AdjustLineBreaks(xmlText);
    LoadXml (xmlText);
  finally
    aStream.Free;
  end;
end;


function XmlIsWellFormed (const strXmlText: string): Boolean;
begin
  Result := True;
  try
    GlobalXslRendering.parseDocument (strXmlText);
  except
    Result := False;
  end;
end;


end.
